@@ -1,10 +1,10 @@
 /*
-  Copyright (c) 2013, 2014, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2013, 2017, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/J is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
   There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
-  this software, see the FLOSS License Exception
+  this software, see the FOSS License Exception
   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
   This program is free software; you can redistribute it and/or modify it under the terms
@@ -27,6 +27,7 @@ import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.util.Properties;
 import java.util.concurrent.Callable;
 import java.util.concurrent.CompletionService;
 import java.util.concurrent.ExecutionException;
@@ -34,10 +35,15 @@ import java.util.concurrent.ExecutorCompletionService;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 
+import javax.sql.XAConnection;
+
 import com.mysql.jdbc.CallableStatement;
+import com.mysql.jdbc.MySQLConnection;
 import com.mysql.jdbc.PreparedStatement;
+import com.mysql.jdbc.ReplicationConnection;
 import com.mysql.jdbc.StatementImpl;
 import com.mysql.jdbc.Util;
+import com.mysql.jdbc.jdbc2.optional.MysqlXADataSource;
 
 import testsuite.BaseTestCase;
 
@@ -1221,7 +1227,7 @@ public class StatementRegressionTest extends BaseTestCase {
         assertFalse(testStep + ".ST:2. Statement.isClosed(): false.", testStatement.isClosed());
 
         assertTrue(testStep + ".ST:3. There should be more ResultSets.", testStatement.getMoreResults()); // closes
-                                                                                                          // testResultSet2
+                                                                                                         // testResultSet2
         testResultSet3 = testStatement.getResultSet();
 
         assertTrue(testStep + ".ST:2. ResultSet.isClosed(): true.", testResultSet1.isClosed());
@@ -1361,13 +1367,12 @@ public class StatementRegressionTest extends BaseTestCase {
         // overridesCloseOnCompletion[n] refers to the effect of connectionProperties[n] on
         // Statement.closeOnCompletion()
         boolean[] overridesCloseOnCompletion = new boolean[] { false, false, true };
-        String[] sampleQueries = new String[] {
-                "SELECT * FROM mysql.help_topic",
-                "SELECT SLEEP(1)",
+        String[] sampleQueries = new String[] { "SELECT * FROM mysql.help_topic", "SELECT SLEEP(1)",
                 "SELECT * FROM mysql.time_zone tz INNER JOIN mysql.time_zone_name tzn ON tz.time_zone_id = tzn.time_zone_id "
                         + "INNER JOIN mysql.time_zone_transition tzt ON tz.time_zone_id = tzt.time_zone_id "
                         + "INNER JOIN mysql.time_zone_transition_type tztt ON tzt.time_zone_id = tztt.time_zone_id "
-                        + "AND tzt.transition_type_id = tztt.transition_type_id ORDER BY tzn.name , tztt.abbreviation , tzt.transition_time", "SELECT 1" };
+                        + "AND tzt.transition_type_id = tztt.transition_type_id ORDER BY tzn.name , tztt.abbreviation , tzt.transition_time",
+                "SELECT 1" };
         int threadCount = sampleQueries.length;
 
         for (int c = 0; c < connectionProperties.length; c++) {
@@ -1466,4 +1471,121 @@ public class StatementRegressionTest extends BaseTestCase {
             }
         }
     }
+
+    /**
+     * Tests fix for Bug#78313 - proxies not handling Object.equals(Object) calls correctly.
+     * 
+     * A reduced version of this test exists in jdbc4.StatementRegressionTest.
+     */
+    public void testBug78313() throws Exception {
+        Connection testConn;
+
+        // Plain connection.
+        testConn = getConnectionWithProps("");
+        assertFalse(testConn.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(testConn.equals(testConn));
+        this.stmt = testConn.createStatement();
+        assertFalse(this.stmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.stmt.equals(this.stmt));
+        this.rs = this.stmt.executeQuery("SELECT 'testBug78313'");
+        assertFalse(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        this.pstmt = testConn.prepareStatement("SELECT 'testBug78313'");
+        assertFalse(this.pstmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.pstmt.equals(this.pstmt));
+        assertFalse(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        testConn.close();
+
+        // Plain connection with proxied result sets.
+        testConn = getConnectionWithProps("statementInterceptors=com.mysql.jdbc.interceptors.ResultSetScannerInterceptor,resultSetScannerRegex=.*");
+        assertFalse(testConn.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(testConn.equals(testConn));
+        this.stmt = testConn.createStatement();
+        assertFalse(this.stmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.stmt.equals(this.stmt));
+        this.rs = this.stmt.executeQuery("SELECT 'testBug78313'");
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        this.pstmt = testConn.prepareStatement("SELECT 'testBug78313'");
+        assertFalse(this.pstmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.pstmt.equals(this.pstmt));
+        this.rs = this.pstmt.executeQuery();
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        testConn.close();
+
+        // Fail-over connection; all JDBC objects are proxied.
+        testConn = getFailoverConnection();
+        assertTrue(testConn.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(testConn.equals(testConn));
+        this.stmt = testConn.createStatement();
+        assertTrue(this.stmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.stmt.equals(this.stmt));
+        this.rs = this.stmt.executeQuery("SELECT 'testBug78313'");
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        this.pstmt = testConn.prepareStatement("SELECT 'testBug78313'");
+        assertTrue(this.pstmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.pstmt.equals(this.pstmt));
+        this.rs = this.pstmt.executeQuery();
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        testConn.close();
+
+        // Load-balanced connection; all JDBC objects are proxied. 
+        testConn = getLoadBalancedConnection();
+        assertTrue(testConn.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(testConn.equals(testConn));
+        this.stmt = testConn.createStatement();
+        assertTrue(this.stmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.stmt.equals(this.stmt));
+        this.rs = this.stmt.executeQuery("SELECT 'testBug78313'");
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        this.pstmt = testConn.prepareStatement("SELECT 'testBug78313'");
+        assertTrue(this.pstmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.pstmt.equals(this.pstmt));
+        this.rs = this.pstmt.executeQuery();
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        testConn.close();
+
+        // Replication connection; all JDBC objects are proxied.
+        testConn = getMasterSlaveReplicationConnection();
+        assertTrue(testConn.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(testConn.equals(testConn));
+        this.stmt = testConn.createStatement();
+        assertTrue(this.stmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.stmt.equals(this.stmt));
+        this.rs = this.stmt.executeQuery("SELECT 'testBug78313'");
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        this.pstmt = testConn.prepareStatement("SELECT 'testBug78313'");
+        assertTrue(this.pstmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.pstmt.equals(this.pstmt));
+        this.rs = this.pstmt.executeQuery();
+        assertTrue(this.rs.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(this.rs.equals(this.rs));
+        testConn.close();
+
+        // XA Connection; unwrapped connections and statements are proxied.
+        MysqlXADataSource xaDs = new MysqlXADataSource();
+        xaDs.setUrl(BaseTestCase.dbUrl);
+        XAConnection xaTestConn = xaDs.getXAConnection();
+        testConn = xaTestConn.getConnection();
+        Connection unwrappedTestConn = testConn.unwrap(com.mysql.jdbc.Connection.class);
+        assertTrue(unwrappedTestConn.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(unwrappedTestConn.equals(unwrappedTestConn));
+        this.stmt = testConn.createStatement();
+        Statement unwrappedStmt = this.stmt.unwrap(com.mysql.jdbc.Statement.class);
+        assertTrue(unwrappedStmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(unwrappedStmt.equals(unwrappedStmt));
+        this.pstmt = testConn.prepareStatement("SELECT 'testBug78313'");
+        Statement unwrappedPstmt = this.pstmt.unwrap(com.mysql.jdbc.Statement.class);
+        assertTrue(unwrappedPstmt.getClass().getName().matches("^(?:com\\.sun\\.proxy\\.)?\\$Proxy\\d*"));
+        assertTrue(unwrappedPstmt.equals(unwrappedPstmt));
+        testConn.close();
+        xaTestConn.close();
+    }
 }
